@@ -1,5 +1,5 @@
 /*  Lzlib - A compression library for lzip files
-    Copyright (C) 2009 Antonio Diaz Diaz.
+    Copyright (C) 2009, 2010 Antonio Diaz Diaz.
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
     As a special exception, you may use this file as part of a free
     software library without restriction.  Specifically, if other files
@@ -31,8 +31,6 @@
 #include <cerrno>
 #include <cstdlib>
 #include <cstring>
-#include <string>
-#include <vector>
 #include <stdint.h>
 
 #include "lzlib.h"
@@ -44,7 +42,7 @@ const Dis_slots dis_slots;
 const Prob_prices prob_prices;
 
 
-int Matchfinder::write_data( uint8_t * const in_buffer, const int in_size ) throw()
+int Matchfinder::write_data( const uint8_t * const in_buffer, const int in_size ) throw()
   {
   if( at_stream_end_ ) return 0;
   const int size = std::min( buffer_size - stream_pos, in_size );
@@ -61,9 +59,8 @@ Matchfinder::Matchfinder( const int dict_size, const int len_limit )
   :
   partial_data_pos( 0 ),
   dictionary_size_( dict_size ),
-  after_size( max_num_trials + max_match_len ),
   buffer_size( ( 2 * std::max( 65536, dictionary_size_ ) ) +
-               max_num_trials + after_size ),
+               before_size + after_size ),
   buffer( new( std::nothrow ) uint8_t[buffer_size] ),
   pos( 0 ),
   cyclic_pos( 0 ),
@@ -88,11 +85,12 @@ Matchfinder::Matchfinder( const int dict_size, const int len_limit )
 void Matchfinder::reset() throw()
   {
   const int size = stream_pos - pos;
-  std::memmove( buffer, buffer + pos, size );
+  if( size > 0 ) std::memmove( buffer, buffer + pos, size );
   partial_data_pos = 0;
   stream_pos -= pos;
   pos = 0;
   cyclic_pos = 0;
+  at_stream_end_ = false;
   for( int i = 0; i < num_prev_positions; ++i ) prev_positions[i] = -1;
   }
 
@@ -105,7 +103,7 @@ bool Matchfinder::move_pos() throw()
     if( pos > stream_pos ) { pos = stream_pos; return false; }
     else
       {
-      const int offset = pos - dictionary_size_ - max_num_trials;
+      const int offset = pos - dictionary_size_ - before_size;
       const int size = stream_pos - offset;
       std::memmove( buffer, buffer + offset, size );
       partial_data_pos += offset;
@@ -310,6 +308,7 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
   trials[0].state = state;
   for( int i = 0; i < num_rep_distances; ++i ) trials[0].reps[i] = reps[i];
 
+  const uint8_t prev_byte = matchfinder[-1];
   const uint8_t cur_byte = matchfinder[0];
   const uint8_t match_byte = matchfinder[-reps[0]-1];
   unsigned int position = matchfinder.data_position();
@@ -319,9 +318,9 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
   trials[1].prev_index = 0;
   trials[1].price = price0( bm_match[state()][pos_state] );
   if( state.is_char() )
-    trials[1].price += literal_encoder.price_symbol( matchfinder[-1], cur_byte );
+    trials[1].price += literal_encoder.price_symbol( prev_byte, cur_byte );
   else
-    trials[1].price += literal_encoder.price_matched( matchfinder[-1], cur_byte, match_byte );
+    trials[1].price += literal_encoder.price_matched( prev_byte, cur_byte, match_byte );
 
   const int match_price = price1( bm_match[state()][pos_state] );
   const int rep_match_price = match_price + price1( bm_rep[state()] );
@@ -355,9 +354,13 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
   }
 
   for( int rep = 0; rep < num_rep_distances; ++rep )
+    {
+    const int price = rep_match_price +
+                      price_rep( rep, state, pos_state );
     for( int len = min_match_len; len <= replens[rep]; ++len )
-      trials[len].update( rep, 0, rep_match_price +
-                                  price_rep( rep, len, state, pos_state ) );
+      trials[len].update( rep, 0, price +
+                                  rep_match_len_encoder.price( len, pos_state ) );
+    }
 
   int cur = 0;
   int num_trials = main_len;
@@ -397,14 +400,15 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
       mtf_reps( cur_trial.dis, cur_trial.reps );
       }
 
+    const uint8_t prev_byte = matchfinder[-1];
     const uint8_t cur_byte = matchfinder[0];
     const uint8_t match_byte = matchfinder[-cur_trial.reps[0]-1];
     const int pos_state = ++position & pos_state_mask;
     int next_price = cur_trial.price + price0( bm_match[cur_trial.state()][pos_state] );
     if( cur_trial.state.is_char() )
-      next_price += literal_encoder.price_symbol( matchfinder[-1], cur_byte );
+      next_price += literal_encoder.price_symbol( prev_byte, cur_byte );
     else
-      next_price += literal_encoder.price_matched( matchfinder[-1], cur_byte, match_byte );
+      next_price += literal_encoder.price_matched( prev_byte, cur_byte, match_byte );
     if( !matchfinder.move_pos() ) return 0;
 
     Trial & next_trial = trials[cur+1];
@@ -430,11 +434,13 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
       while( len < len_limit && data[len] == data[len-dis] ) ++len;
       if( len >= min_match_len )
         {
+        const int price = rep_match_price +
+                          price_rep( rep, cur_trial.state, pos_state );
         while( num_trials < cur + len )
           trials[++num_trials].price = infinite_price;
         for( ; len >= min_match_len; --len )
-          trials[cur+len].update( rep, cur, rep_match_price +
-                                  price_rep( rep, len, cur_trial.state, pos_state ) );
+          trials[cur+len].update( rep, cur, price +
+                                  rep_match_len_encoder.price( len, pos_state ) );
         }
       }
 
@@ -448,7 +454,7 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
       while( num_trials < cur + newlen )
         trials[++num_trials].price = infinite_price;
 
-      for( int len = newlen; len >= min_match_len; --len )
+      for( int len = min_match_len; len <= newlen; ++len )
         trials[cur+len].update( match_distances[len] + num_rep_distances, cur,
                                 normal_match_price +
                                 price_pair( match_distances[len], len, pos_state ) );
@@ -457,7 +463,7 @@ int LZ_encoder::best_pair_sequence( const int reps[num_rep_distances],
   }
 
 
-     // Sync Flush mark => (dis == 0xFFFFFFFF, len == min_match_len+1)
+     // Sync Flush mark => (dis == 0xFFFFFFFF, len == min_match_len + 1)
 bool LZ_encoder::sync_flush()
   {
   if( member_finished_ || range_encoder.free_bytes() < max_marker_size )
@@ -505,7 +511,6 @@ LZ_encoder::LZ_encoder( Matchfinder & mf, const File_header & header,
   literal_encoder(),
   num_dis_slots( 2 * File_header::real_bits( matchfinder.dictionary_size() - 1 ) ),
   fill_counter( 0 ),
-  prev_byte( 0 ),
   member_finished_( false )
   {
   for( int i = 0; i < num_rep_distances; ++i ) rep_distances[i] = 0;
@@ -522,15 +527,15 @@ bool LZ_encoder::encode_member( const bool finish )
   if( range_encoder.member_position() >= member_size_limit )
     { if( full_flush() ) { member_finished_ = true; } return true; }
 
-  // copy first byte
+  // encode first byte
   if( matchfinder.data_position() == 0 && !matchfinder.finished() )
     {
     if( matchfinder.available_bytes() < 4 && !matchfinder.at_stream_end() )
       return true;
     range_encoder.encode_bit( bm_match[state()][0], 0 );
+    const uint8_t prev_byte = 0;
     const uint8_t cur_byte = matchfinder[0];
     literal_encoder.encode( range_encoder, prev_byte, cur_byte );
-    prev_byte = cur_byte;
     crc32.update( crc_, cur_byte );
     if( !move_pos( 1 ) ) return false;
     }
@@ -560,16 +565,16 @@ bool LZ_encoder::encode_member( const bool finish )
       range_encoder.encode_bit( bm_match[state()][pos_state], !bit );
       if( bit )
         {
+        const uint8_t prev_byte = matchfinder[-ahead-1];
         const uint8_t cur_byte = matchfinder[-ahead];
         if( state.is_char() )
           literal_encoder.encode( range_encoder, prev_byte, cur_byte );
         else
           {
-          const uint8_t match_byte = matchfinder[-rep_distances[0]-1-ahead];
+          const uint8_t match_byte = matchfinder[-ahead-rep_distances[0]-1];
           literal_encoder.encode_matched( range_encoder, prev_byte, match_byte, cur_byte );
           }
         state.set_char();
-        prev_byte = cur_byte;
         }
       else
         {
@@ -600,7 +605,6 @@ bool LZ_encoder::encode_member( const bool finish )
           encode_pair( dis - num_rep_distances, len, pos_state );
           state.set_match();
           }
-        prev_byte = matchfinder[len-1-ahead];
         }
       for( int j = 0; j < len; ++j )
         crc32.update( crc_, matchfinder[j-ahead] );
