@@ -1,5 +1,5 @@
 /*  Lzlib - A compression library for lzip files
-    Copyright (C) 2009 Antonio Diaz Diaz.
+    Copyright (C) 2009, 2010 Antonio Diaz Diaz.
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
     As a special exception, you may use this file as part of a free
     software library without restriction.  Specifically, if other files
@@ -55,6 +55,7 @@ public:
 
 extern const Dis_slots dis_slots;
 
+
 class Prob_prices
   {
   int data[bit_model_total >> 2];
@@ -79,6 +80,7 @@ public:
 
 extern const Prob_prices prob_prices;
 
+
 inline int price0( const Bit_model & bm ) throw()
   { return prob_prices[bm.probability]; }
 
@@ -88,6 +90,7 @@ inline int price1( const Bit_model & bm ) throw()
 inline int price_bit( const Bit_model & bm, const int bit ) throw()
   { if( bit ) return price1( bm ); else return price0( bm ); }
 
+
 inline int price_symbol( const Bit_model bm[], int symbol, const int num_bits ) throw()
   {
   symbol |= ( 1 << num_bits );
@@ -101,6 +104,7 @@ inline int price_symbol( const Bit_model bm[], int symbol, const int num_bits )
   return price;
   }
 
+
 inline int price_symbol_reversed( const Bit_model bm[], int symbol,
                                   const int num_bits ) throw()
   {
@@ -116,6 +120,7 @@ inline int price_symbol_reversed( const Bit_model bm[], int symbol,
   return price;
   }
 
+
 inline int price_matched( const Bit_model bm[], const int symbol,
                           const int match_byte ) throw()
   {
@@ -145,7 +150,11 @@ inline int price_matched( const Bit_model bm[], const int symbol,
 
 class Matchfinder
   {
-  enum { num_prev_positions4 = 1 << 20,
+  enum { // bytes to keep in buffer before dictionary
+         before_size = max_num_trials + 1,
+         // bytes to keep in buffer after pos
+         after_size = max_num_trials + max_match_len,
+         num_prev_positions4 = 1 << 20,
          num_prev_positions3 = 1 << 18,
          num_prev_positions2 = 1 << 16,
          num_prev_positions = num_prev_positions4 + num_prev_positions3 +
@@ -153,7 +162,6 @@ class Matchfinder
 
   long long partial_data_pos;
   const int dictionary_size_;	// bytes to keep in buffer before pos
-  const int after_size;		// bytes to keep in buffer after pos
   const int buffer_size;
   uint8_t * const buffer;
   int pos;
@@ -178,7 +186,8 @@ public:
   int dictionary_size() const throw() { return dictionary_size_; }
   void flushing( const bool b ) throw() { at_stream_end_ = b; }
   bool finished() const throw() { return at_stream_end_ && pos >= stream_pos; }
-  int free_bytes() const throw() { return buffer_size - stream_pos; }
+  int free_bytes() const throw()
+    { if( at_stream_end_ ) return 0; return buffer_size - stream_pos; }
   int match_len_limit() const throw() { return match_len_limit_; }
   const uint8_t * ptr_to_current_pos() const throw() { return buffer + pos; }
 
@@ -207,7 +216,7 @@ public:
     return i;
     }
 
-  int write_data( uint8_t * const in_buffer, const int in_size ) throw();
+  int write_data( const uint8_t * const in_buffer, const int in_size ) throw();
   void reset() throw();
   bool move_pos() throw();
   int longest_match_len( int * const distances = 0 ) throw();
@@ -226,14 +235,14 @@ class Range_encoder : public Circular_buffer
   void shift_low()
     {
     const uint32_t carry = low >> 32;
-    if( low < 0xFF000000 || carry == 1 )
+    if( low < 0xFF000000LL || carry == 1 )
       {
       put_byte( cache + carry );
       for( ; ff_count > 0; --ff_count ) put_byte( 0xFF + carry );
       cache = low >> 24;
       }
     else ++ff_count;
-    low = ( low & 0x00FFFFFF ) << 8;
+    low = ( low & 0x00FFFFFFLL ) << 8;
     }
 
 public:
@@ -414,14 +423,14 @@ class LZ_encoder
   {
   enum { dis_align_mask = dis_align_size - 1,
          infinite_price = 0x0FFFFFFF,
-         max_marker_size = 15,
+         max_marker_size = 16,
          num_rep_distances = 4 };	// must be 4
 
   struct Trial
     {
     State state;
     int dis;
-    int prev_index;
+    int prev_index;	// index of prev trial in trials[]
     int price;		// dual use var; cumulative price, match length
     int reps[num_rep_distances];
     void update( const int d, const int p_i, const int pr ) throw()
@@ -459,7 +468,6 @@ class LZ_encoder
   int align_price_count;
   int fill_counter;
   State state;
-  uint8_t prev_byte;
   bool member_finished_;
 
   void fill_align_prices() throw();
@@ -487,25 +495,18 @@ class LZ_encoder
     return price0( bm_rep0[state()] ) + price0( bm_len[state()][pos_state] );
     }
 
-  int price_rep( const int rep, const int len, const State & state,
+  int price_rep( const int rep, const State & state,
                  const int pos_state ) const throw()
     {
-    int price = rep_match_len_encoder.price( len, pos_state );
-    if( rep == 0 )
-      {
-      price += price0( bm_rep0[state()] );
-      price += price1( bm_len[state()][pos_state] );
-      }
+    if( rep == 0 ) return price0( bm_rep0[state()] ) +
+                          price1( bm_len[state()][pos_state] );
+    int price = price1( bm_rep0[state()] );
+    if( rep == 1 )
+      price += price0( bm_rep1[state()] );
     else
       {
-      price += price1( bm_rep0[state()] );
-      if( rep == 1 )
-        price += price0( bm_rep1[state()] );
-      else
-        {
-        price += price1( bm_rep1[state()] );
-        price += price_bit( bm_rep2[state()], rep - 2 );
-        }
+      price += price1( bm_rep1[state()] );
+      price += price_bit( bm_rep2[state()], rep - 2 );
       }
     return price;
     }
